<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="text" encoding="iso-8859-1"/>
  <xsl:template match="alf">//spu_<xsl:value-of select="//PROJECT_NAME"/>.c: source file for accelerator node program
#include &lt;alf_accel.h&gt;
#include &lt;stdio.h&gt;
#include "common.h"

int comp_kernel(void *p_task_context,
                void *p_parm_ctx_buffer,
                void *p_input_buffer,
                void *p_output_buffer,
                void *p_inout_buffer,
                unsigned int current_count,
                unsigned int total_count)
{
 <xsl:for-each select="buffer">
 <xsl:text>	</xsl:text><xsl:value-of select="ELEMENT_TYPE"/> *<xsl:value-of select="VARIABLE_NAME"/>;
 </xsl:for-each>
	my_param_t *p_param = (my_param_t *) p_parm_ctx_buffer;
<xsl:for-each select="buffer"><xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_INPUT')"> 
<xsl:text>	</xsl:text><xsl:value-of select="VARIABLE_NAME"/> = (<xsl:value-of select="ELEMENT_TYPE"/>*) ((char*)p_input_buffer + p_param->c.buffer_offset[<xsl:number value="position()-1"/>]);
</xsl:if><xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_OUTPUT')">
<xsl:text>	</xsl:text><xsl:value-of select="VARIABLE_NAME"/> = (<xsl:value-of select="ELEMENT_TYPE"/>*) ((char*)p_output_buffer + p_param->c.buffer_offset[<xsl:number value="position()-1"/>]);
</xsl:if>
</xsl:for-each><xsl:if test = "number(///BUFFER_NUMBER) = 0">
</xsl:if>
	//TODO: Code your computing kernel here
	//e.g. printf("%s\n", p_param->c.msg);
	
	return 0;
}
<xsl:if test = "number(///BUFFER_NUMBER) > 0">
#ifdef ACCELERATOR_PARTITION

#define MAX_DMA_ENTRY_SIZE 0x4000

#define alf_generate_dt_list(p_dt_list_buffer, p_param, index, buf_type, element_type, offset)    \
{                                                                                                 \
    unsigned int size;                                                                            \
    alf_data_addr64_t ad;                                                                         \
                                                                                                  \
    ALF_ACCEL_DTL_BEGIN(p_dt_list_buffer, buf_type, offset);                                      \
    size = p_param->c.size[index] * sizeof(element_type);                                         \
    ad = p_param->c.ea[index];                                                                    \
                                                                                                  \
    while(size > MAX_DMA_ENTRY_SIZE)                                                              \
    {                                                                                             \
        ALF_ACCEL_DTL_ENTRY_ADD(p_dt_list_buffer,                                                 \
        MAX_DMA_ENTRY_SIZE, ALF_DATA_BYTE, ad);                                                   \
        ad += MAX_DMA_ENTRY_SIZE;                                                                 \
        size -= MAX_DMA_ENTRY_SIZE;                                                               \
    }                                                                                             \
                                                                                                  \
    ALF_ACCEL_DTL_ENTRY_ADD(p_dt_list_buffer,size,ALF_DATA_BYTE,ad);                              \
    ALF_ACCEL_DTL_END(p_dt_list_buffer);                                                          \
}
#endif</xsl:if>

int input_prepare(void* p_task_context __attribute__ ((unused)),
                  void *p_parm_ctx_buffer,
                  void *p_dtl,
                  unsigned int current_count __attribute__ ((unused)),
                  unsigned int total_count __attribute__ ((unused)))
{<xsl:if test = "number(///BUFFER_NUMBER) > 0">
	my_param_t *p_param = (my_param_t *) p_parm_ctx_buffer;</xsl:if>
<xsl:if test = "count(buffer/BUFFER_TYPE[text() = 'ALF_BUFFER_INPUT']) > 0">
#ifdef ACCELERATOR_PARTITION<xsl:for-each select="buffer"><xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_INPUT')">
	//Wizard generated dt list for <xsl:value-of select="string(VARIABLE_NAME)"/> 
	alf_generate_dt_list(p_dtl, p_param, <xsl:number value="position()-1"/>, ALF_BUF_IN, <xsl:value-of select="ELEMENT_TYPE"/>, p_param->c.buffer_offset[<xsl:number value="position()-1"/>]);</xsl:if>
</xsl:for-each>
#endif</xsl:if>
	//TODO: Code your prepare_input_list here

	return 0;
}

int output_prepare(void *p_task_context, 
                   void *p_parm_ctx_buffer,
                   void *p_dtl,
                   unsigned int current_count,
                   unsigned int total_count)
{<xsl:if test = "number(///BUFFER_NUMBER) > 0">
	my_param_t *p_param = (my_param_t *) p_parm_ctx_buffer;</xsl:if>
<xsl:if test = "count(buffer/BUFFER_TYPE[text() = 'ALF_BUFFER_OUTPUT']) > 0">
#ifdef ACCELERATOR_PARTITION<xsl:for-each select="buffer"><xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_OUTPUT')">
	//Wizard generated dt list for <xsl:value-of select="string(VARIABLE_NAME)"/>
	alf_generate_dt_list(p_dtl, p_param, <xsl:number value="position()-1"/>, ALF_BUF_OUT, <xsl:value-of select="ELEMENT_TYPE"/>, p_param->c.buffer_offset[<xsl:number value="position()-1"/>]);</xsl:if>
</xsl:for-each>
#endif</xsl:if>
	//TODO: Code your prepare_input_list here

	return 0;
}

int task_context_setup(void *p_task_context)
{
	//TODO: Code your task_context_setup here

	return 0;
}

int task_context_merge(void* p_dst_task_context, void* p_task_context)
{
	//TODO: Code your task_context_merge here

	return 0;
}

ALF_ACCEL_EXPORT_API_LIST_BEGIN
	ALF_ACCEL_EXPORT_API("kernel", comp_kernel)
	ALF_ACCEL_EXPORT_API("input", input_prepare)
	ALF_ACCEL_EXPORT_API("output", output_prepare)
	ALF_ACCEL_EXPORT_API("ctx setup", task_context_setup)
	ALF_ACCEL_EXPORT_API("ctx merge", task_context_merge)
ALF_ACCEL_EXPORT_API_LIST_END
</xsl:template>
</xsl:stylesheet> 
