<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="text" encoding="iso-8859-1"/>
  <xsl:template match="alf">//ppu_<xsl:value-of select="//PROJECT_NAME"/>.c: source file for host node program.
#include &lt;stdio.h&gt;
#include &lt;stdlib.h&gt;
#include &lt;string.h&gt;
#include &lt;libgen.h&gt;
#include &lt;unistd.h&gt;
#include &lt;alf.h&gt;
#include "common.h"
#include "ppu_<xsl:value-of select="//PROJECT_NAME"/>.h"

#define MY_ALIGN(_my_var_type, _my_var_def_, _my_al_)  _my_var_type _my_var_def_  \
__attribute__((__aligned__(_my_al_)))

#define PATH_BUF_SIZE 1024                // Max length of path to PPU image

char spu_image_path[] = ".";              // Used to hold the complete path to SPU image
char library_name[PATH_BUF_SIZE + 1];     // Used to hold the name of spu library
char spu_image_name[]  = "spu_<xsl:value-of select="//PROJECT_NAME"/>";
char kernel_name[]     = "comp_kernel";
char input_dtl_name[]  = "input_prepare";
char output_dtl_name[] = "output_prepare";
char ctx_setup_name[]  = "task_context_setup";
char ctx_merge_name[]  = "task_context_merge";
<xsl:for-each select="buffer">
MY_ALIGN(<xsl:value-of select="ELEMENT_TYPE"/>, <xsl:value-of select="VARIABLE_NAME"/><xsl:if test = "number(DIMENSION_SIZE_Z) > 0">[<xsl:value-of select="number(DIMENSION_SIZE_Z)"/>]</xsl:if><xsl:if test = "number(DIMENSION_SIZE_Y) > 0">[<xsl:value-of select="number(DIMENSION_SIZE_Y)"/>]</xsl:if><xsl:if test = "number(DIMENSION_SIZE_X) > 0">[<xsl:value-of select="number(DIMENSION_SIZE_X)"/>]</xsl:if>, 16);</xsl:for-each>

int main(int argc, char * argv[])
{
	alf_sys_config_t_CBEA cell_config_parms;
	alf_handle_t alf_handle;
	alf_task_desc_handle_t desc_info_handle;
	alf_task_handle_t task_handle;
	alf_wb_handle_t wb_handle;
	unsigned int nodes;
	int err;
	char *dir = NULL;
	MY_ALIGN(my_param_t, param, 128);
<xsl:if test = "number(///BUFFER_NUMBER) > 0">
	unsigned int i;
	unsigned int input_buffer_size = 0u;
	unsigned int output_buffer_size = 0u;
	unsigned int dt_entry = 1u;
	unsigned int size[WZ_ARRAY_NUM];
#ifndef  ACCELERATOR_PARTITION
	char* ea[WZ_ARRAY_NUM];
#endif	
	dt_entry = <xsl:value-of select="number(///DIMENSION_SIZE_X)"/> / <xsl:value-of select="///DISTRIBUTION_SIZE_X"/>;<xsl:if test = "number(///DIMENSION_SIZE_Y) > 0">
	dt_entry = dt_entry * (<xsl:value-of select="number(///DIMENSION_SIZE_Y)"/> / <xsl:value-of select="///DISTRIBUTION_SIZE_Y"/>);</xsl:if><xsl:if test = "number(///DIMENSION_SIZE_Z) > 0">
	dt_entry = dt_entry * (<xsl:value-of select="number(///DIMENSION_SIZE_Z)"/> / <xsl:value-of select="///DISTRIBUTION_SIZE_Z"/>);</xsl:if>
	<xsl:for-each select="buffer">
        
	//Partitioning buffer size for <xsl:value-of select="VARIABLE_NAME"/>
	<xsl:if test="number(NUM_DIMENSION) = 1">
	param.c.size[<xsl:number value="position()-1"/>] = <xsl:value-of select="number(DISTRIBUTION_SIZE_X)"/>;</xsl:if><xsl:if test="number(NUM_DIMENSION) = 2">
	param.c.size[<xsl:number value="position()-1"/>] = <xsl:value-of select="number(DISTRIBUTION_SIZE_X)"/> * <xsl:value-of select="number(DISTRIBUTION_SIZE_Y)"/>;</xsl:if><xsl:if test="number(NUM_DIMENSION) = 3">
	param.c.size[<xsl:number value="position()-1"/>] = <xsl:value-of select="number(DISTRIBUTION_SIZE_X)"/> * <xsl:value-of select="number(DISTRIBUTION_SIZE_Y)"/> * <xsl:value-of select="number(DISTRIBUTION_SIZE_X)"/>;
	</xsl:if>
	size[<xsl:number value="position()-1"/>] = param.c.size[<xsl:number value="position()-1"/>] * sizeof(<xsl:value-of select="ELEMENT_TYPE"/>);<xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_INPUT')">
	param.c.buffer_offset[<xsl:number value="position()-1"/>] = input_buffer_size; 
	input_buffer_size += size[<xsl:number value="position()-1"/>];</xsl:if><xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_OUTPUT')">
	param.c.buffer_offset[<xsl:number value="position()-1"/>] = output_buffer_size; 
	output_buffer_size += size[<xsl:number value="position()-1"/>];
        </xsl:if></xsl:for-each></xsl:if>

	// Absolute path for spu .so file
	dir = dirname(argv[0]);

	if (dir == NULL)
	{
		fprintf(stderr, "Error in query working directory.\n");
		cell_config_parms.library_path = spu_image_path;
	}
	else
	{
		cell_config_parms.library_path = dir;
	}

	sprintf(library_name, "lib<xsl:value-of select="//PROJECT_NAME"/>.so");
        
	alf_init(&amp;cell_config_parms, &amp;alf_handle);
	if ((err = alf_query_system_info(alf_handle, ALF_QUERY_NUM_ACCEL, ALF_ACCEL_TYPE_SPE, &amp;nodes)) &lt; 0)
	{
		fprintf(stderr, "Error in query system information.\n");
		return (-1);
	}
<xsl:if test = "number(//EXP_ACCEL_NUM) > 0">
	if (nodes &lt; <xsl:value-of select="number(//EXP_ACCEL_NUM)"/>)
	{
		fprintf(stderr, "Not enough accelerator nodes.\n");
		return -1;
	}
        
	nodes = <xsl:value-of select="number(//EXP_ACCEL_NUM)"/>;
</xsl:if>
	alf_num_instances_set (alf_handle, nodes);
	alf_task_desc_create (alf_handle, ALF_ACCEL_TYPE_SPE, &amp;desc_info_handle);
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_WB_PARM_CTX_BUF_SIZE, sizeof(my_param_t));<xsl:if test = "number(///BUFFER_NUMBER) > 0">
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_WB_IN_BUF_SIZE, input_buffer_size);
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_WB_OUT_BUF_SIZE, output_buffer_size);
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_WB_INOUT_BUF_SIZE, 0);
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_NUM_DTL_ENTRIES, dt_entry);</xsl:if><xsl:if test = "number(///BUFFER_NUMBER) =0">
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_WB_IN_BUF_SIZE, 0);
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_WB_OUT_BUF_SIZE, 0);
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_WB_INOUT_BUF_SIZE, 0);
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_NUM_DTL_ENTRIES, 1);</xsl:if>
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_TSK_CTX_SIZE, 0);
 	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_MAX_STACK_SIZE, <xsl:value-of select="//STACK_SIZE"/>);
#ifdef ACCELERATOR_PARTITION
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_PARTITION_ON_ACCEL, 1);
#else	
	alf_task_desc_set_int32 (desc_info_handle, ALF_TASK_DESC_PARTITION_ON_ACCEL, 0);
#endif
	alf_task_desc_set_int64 (desc_info_handle, ALF_TASK_DESC_ACCEL_IMAGE_REF_L, (unsigned long long)spu_image_name);
	alf_task_desc_set_int64 (desc_info_handle, ALF_TASK_DESC_ACCEL_LIBRARY_REF_L, (unsigned long long)library_name);
	alf_task_desc_set_int64 (desc_info_handle, ALF_TASK_DESC_ACCEL_KERNEL_REF_L, (unsigned long long)kernel_name);
	alf_task_desc_set_int64 (desc_info_handle, ALF_TASK_DESC_ACCEL_INPUT_DTL_REF_L, (unsigned long long)input_dtl_name);
	alf_task_desc_set_int64 (desc_info_handle, ALF_TASK_DESC_ACCEL_OUTPUT_DTL_REF_L, (unsigned long long)output_dtl_name);
	alf_task_desc_set_int64 (desc_info_handle, ALF_TASK_DESC_ACCEL_CTX_SETUP_REF_L, (unsigned long long)ctx_setup_name);
	alf_task_desc_set_int64 (desc_info_handle, ALF_TASK_DESC_ACCEL_CTX_MERGE_REF_L, (unsigned long long)ctx_merge_name);
<xsl:if test = "number(///BUFFER_NUMBER) > 0">
#ifdef ACCELERATOR_PARTITION<xsl:for-each select="buffer">
	PTR_TO_ADDR64(<xsl:value-of select="VARIABLE_NAME"/>, param.c.ea[<xsl:number value="position()-1"/>]);</xsl:for-each>
#else<xsl:for-each select="buffer">
	ea[<xsl:number value="position()-1"/>] = (char*) <xsl:value-of select="VARIABLE_NAME"/>;</xsl:for-each>
#endif</xsl:if>
	alf_task_create(desc_info_handle, NULL, 0, 0, 1, &amp;task_handle);
<xsl:if test = "number(//BUFFER_NUMBER) > 0">
	//TODO: Code your application here, prepare input or output buffer
	
	
	for (i = 0; i &lt; dt_entry; ++i)
	{
		alf_wb_create(task_handle, ALF_WB_SINGLE, 0, &amp;wb_handle);
#ifdef ACCELERATOR_PARTITION
		alf_wb_parm_add(wb_handle, (void *)(&amp;param), sizeof(my_param_t), ALF_DATA_BYTE, 0);<xsl:for-each select="buffer">
		//Wizard generated parameter for <xsl:value-of select="VARIABLE_NAME"/>
		param.c.ea[<xsl:number value="position()-1"/>] += (unsigned long long)size[<xsl:number value="position()-1"/>];</xsl:for-each>
#else<xsl:for-each select="buffer">
		//Wizard generated io_buffer entry for <xsl:value-of select="VARIABLE_NAME"/><xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_INPUT')">
		alf_wb_dtl_begin(wb_handle, ALF_BUF_IN, param.c.buffer_offset[<xsl:number value="position()-1"/>]);</xsl:if><xsl:if test = "contains(BUFFER_TYPE, 'ALF_BUFFER_OUTPUT')">
		alf_wb_dtl_begin(wb_handle, ALF_BUF_OUT, param.c.buffer_offset[<xsl:number value="position()-1"/>]);</xsl:if>
		alf_wb_dtl_entry_add(wb_handle, ea[<xsl:number value="position()-1"/>], size[<xsl:number value="position()-1"/>], ALF_DATA_BYTE);
		alf_wb_dtl_end(wb_handle);
		ea[<xsl:number value="position()-1"/>] += size[<xsl:number value="position()-1"/>];</xsl:for-each>
		alf_wb_parm_add(wb_handle, (void *)(&amp;param), sizeof(my_param_t), ALF_DATA_BYTE, 0);
#endif
		alf_wb_enqueue(wb_handle);
	}
</xsl:if> <!--- if buffer-number > 0 -->
<xsl:if test = "number(///BUFFER_NUMBER) = 0">
	strcpy(param.c.msg, "Hello World!");
	alf_wb_create(task_handle, ALF_WB_SINGLE, 0, &amp;wb_handle);
	alf_wb_parm_add(wb_handle, (void *)(&amp;param), sizeof(my_param_t), ALF_DATA_BYTE, 0);
	alf_wb_enqueue(wb_handle);
</xsl:if>
	alf_task_finalize (task_handle);
	
	//TODO: Code your application here
	
	
	alf_task_wait(task_handle, -1);
	alf_task_destroy(task_handle);
	alf_exit(alf_handle, ALF_EXIT_POLICY_WAIT, -1);
	
	return 0;
}

</xsl:template>
</xsl:stylesheet>

